@@ -3,10 +3,9 @@
 #include "constant.h"
 #include <cstddef>
 #include "psqn-misc.h"
-#include "memory.h"
 #include "lp.h"
 #include "intrapolate.h"
-#include <memory>
+#include <vector>
 #include <algorithm>
 #include <cmath>
 #include <limits>
@@ -15,6 +14,16 @@ namespace PSQN {
 using std::abs;
 using std::sqrt;
 
+namespace bfgs_defaults {
+constexpr double rel_tol{.00000001},
+                 c1{.0001},
+                 c2{.9},
+                 gr_tol{-1},
+                 abs_tol{-1};
+constexpr psqn_uint max_it{100};
+constexpr int trace{0};
+}
+
 /** base problem class to pass to optimization method. */
 class problem {
 public:
@@ -28,10 +37,24 @@ public:
   virtual ~problem() = default;
 };
 
+/**
+ returns the required working memory for bfgs.
+ @param n_ele the number of parameters.
+ */
+constexpr psqn_uint bfgs_n_wmem(psqn_uint const n_ele){
+  return 7 * n_ele + (n_ele * (n_ele + 1)) / 2;
+}
+
+/// overload that takes a problem
+inline psqn_uint bfgs_n_wmem(problem const &prob){
+  return bfgs_n_wmem(prob.size());
+}
+
 /**
  minimizes a function.
  @param prob problem with function to be minimized.
  @param val starting value. Result on return.
+ @param mem working memory.
  @param rel_eps relative convergence threshold.
  @param max_it maximum number of iterations.
  @param c1,c2 thresholds for Wolfe condition.
@@ -39,20 +62,24 @@ public:
  @param trace controls the amount of tracing information.
  @param gr_tol convergence tolerance for the Euclidean norm of the gradient. A negative
  value yields no check.
+ @param abs_tol absolute convergence threshold. A negative value yields not
+ check.
  */
 template<class Reporter = dummy_reporter,
          class interrupter = dummy_interrupter>
 optim_info bfgs(
-    problem &prob, double *val, double const rel_eps = .00000001,
-    psqn_uint const max_it = 100, double const c1 = .0001,
-    double const c2 = .9, int const trace = 0L, double const gr_tol = -1){
+    problem &prob, double *val, double * const mem,
+    double const rel_eps = bfgs_defaults::rel_tol,
+    psqn_uint const max_it = bfgs_defaults::max_it,
+    double const c1 = bfgs_defaults::c1, double const c2 = bfgs_defaults::c2,
+    int const trace = bfgs_defaults::trace,
+    double const gr_tol = bfgs_defaults::gr_tol,
+    double const abs_tol = bfgs_defaults::abs_tol){
   // allocate the memory we need
   /* non-const due to
    *    https://www.mail-archive.com/gcc-bugs@gcc.gnu.org/msg531670.html */
   psqn_uint n_ele = prob.size();
-  std::unique_ptr<double[]>
-    mem(new double[7 * n_ele + (n_ele * (n_ele + 1)) / 2]);
-  double * PSQN_RESTRICT const v_old  = mem.get(),
+  double * PSQN_RESTRICT const v_old  = mem,
          * PSQN_RESTRICT const gr     = v_old  + n_ele,
          * PSQN_RESTRICT const gr_old = gr     + n_ele,
          * PSQN_RESTRICT const s      = gr_old + n_ele,
@@ -133,6 +160,7 @@ optim_info bfgs(
       double const f0, double * PSQN_RESTRICT x0, double * PSQN_RESTRICT gr0,
       double * PSQN_RESTRICT dir, double &fnew) -> bool {
     double * const x_mem = wrk;
+    double const forg{fnew};
 
     // declare 1D functions
     auto psi = [&](double const alpha) -> double {
@@ -229,7 +257,10 @@ optim_info bfgs(
       if(fi > f0 + c1 * ai * dpsi_zero or (found_ok_prev and fi > fold)){
         intrapolate inter(f0, dpsi_zero, ai, fi);
         bool const out = zoom(a_prev, ai, inter);
-        lp::copy(x0, x_mem, n_ele);
+        if(out || (std::isfinite(fnew) && fnew < forg))
+          lp::copy(x0, x_mem, n_ele);
+        else
+          fnew = forg; // we did not change x0
         return out;
       }
 
@@ -259,7 +290,10 @@ optim_info bfgs(
           return intrapolate(f0, dpsi_zero, ai, fi);
         })();
         bool const out = zoom(ai, a_prev, inter);
-        lp::copy(x0, x_mem, n_ele);
+        if(out || (std::isfinite(fnew) && fnew < forg))
+          lp::copy(x0, x_mem, n_ele);
+        else
+          fnew = forg; // we did not change x0
         return out;
       }
 
@@ -302,10 +336,14 @@ optim_info bfgs(
          std::min(n_print, n_ele));
     }
 
-    bool const has_converged =
-      abs(fval - fval_old) < rel_eps * (abs(fval_old) + rel_eps) &&
-      // TODO: implement something like BLAS nrm2 function
-      (gr_tol <= 0 || lp::vec_dot<false>(gr, n_ele) < gr_tol * gr_tol);
+    double const err{abs(fval - fval_old)};
+    bool const passed_rel_tol{err < rel_eps * (abs(fval_old) + rel_eps)},
+               passed_abs_tol{abs_tol < 0 || err < abs_tol},
+               passed_gr_tol
+      {gr_tol < 0 || lp::vec_dot<false>(gr, n_ele) < gr_tol * gr_tol};
+    bool const has_converged
+      {n_line_search_fail < 1 &&
+        passed_rel_tol && passed_abs_tol && passed_gr_tol};
     if(has_converged){
       info = info_code::converged;
       break;
@@ -322,6 +360,21 @@ optim_info bfgs(
 
   return { fval, info, n_eval, n_grad, 0 };
 }
+
+/// overload that allocates working memory
+template<class Reporter = dummy_reporter,
+         class interrupter = dummy_interrupter>
+optim_info bfgs(
+    problem &prob, double *val, double const rel_eps = bfgs_defaults::rel_tol,
+    psqn_uint const max_it = bfgs_defaults::max_it,
+    double const c1 = bfgs_defaults::c1, double const c2 = bfgs_defaults::c2,
+    int const trace = bfgs_defaults::trace,
+    double const gr_tol = bfgs_defaults::gr_tol,
+    double const abs_tol = bfgs_defaults::abs_tol){
+  std::vector<double> mem(bfgs_n_wmem(prob));
+  return bfgs<Reporter, interrupter>
+    (prob, val, mem.data(), rel_eps, max_it, c1, c2, trace, gr_tol, abs_tol);
+}
 } // namespace PSQN
 
 #endif
